@@ -85,7 +85,7 @@ update.eta <- function(eta, R_X, R_Y, R_Z, rbetas, G, nyear, n_eta, cons.logLik)
     }
     rm(product, gr)
     chol.H <- chol(H)
-    log.p.eta <- -(length(eta)/2) * log(2 * pi) - sum(log(diag(G.chol))) - 0.5 * crossprod(eta, G.inv) %*% eta
+    log.p.eta <- -(length(eta)/2) * log(2 * pi) - sum(log(diag(G.chol))) - 0.5 * crossprod(eta, as(G.inv,"generalMatrix")) %*% eta
     log.p.r <- sum(pnorm(as.vector((-1)^(1 - R_Y) * (R_X %*% rbetas + R_Z %*% eta)), log.p = TRUE))
     res <- var.eta
     attr(res, "likelihood") <- as.vector(cons.logLik + log.p.eta  + log.p.r - 0.5 * (2 * sum(log(diag(chol.H)))))
@@ -142,6 +142,70 @@ update.rbetas.first.order <- function(eta, rbetas, R_X, R_Y, R_Z, n_eta) {
     rm( scrbetas, product)
     rbetas
     }
+    
+    
+    Score <- function(thetas) {
+   
+    if(home.field){
+      rbetas <- thetas[1]
+    if(dim(R_X)[2]==2) rbetas<-c(rbetas,0)
+    }else{
+    rbetas <- rep(0,ncol(R_X))
+    }
+    if(home.field){
+    G <- thetas[2]
+    }else{
+     G <- thetas[1]
+    }
+    G<-G*Diagonal(length(teams))
+    #update.eta returns new var.eta with eta and likelihood as attr()
+    new.eta <- update.eta(eta = eta.hat, R_X = R_X, R_Y = R_Y, R_Z = R_Z,rbetas = rbetas, G = G, n_eta = n_eta, cons.logLik = cons.logLik)
+    eta <- attr(new.eta, "eta")
+    eta.hat<-eta
+    var.eta <- var.eta.hat <- new.eta
+    eta.hat <- as.vector(eta)
+    temp_mat <- var.eta.hat + tcrossprod(eta.hat, eta.hat)
+   # temp_mat_R <- attr(new.eta, "h.inv") + tcrossprod(eta.hat,
+    #        eta.hat)
+    rm(new.eta)
+
+   if(home.field) score.r <- Sc.rbetas.f.first.order(eta = eta, rbetas = rbetas, R_X = R_X, R_Y = R_Y, R_Z = R_Z)[1]          
+    
+     gam_t_sc <- list()
+        index1 <- 0
+        score.G <- Matrix(0, 0, 0)
+         gam_t_sc <- matrix(0, 1,1)
+         index2 <- c(1)
+         for (k in 1:nrow(G)) {
+                gam_t_sc <- gam_t_sc + temp_mat[(index2):(index2 + 
+                  0), (index2):(index2 + 0)]
+                index2 <- index2 + 1
+            }
+            gam_t <- G[1, 1]
+            sv_gam_t <- chol2inv(chol(gam_t))
+        der <- -0.5 * (nrow(G) * sv_gam_t - sv_gam_t %*% 
+                gam_t_sc %*% sv_gam_t)
+            if (is.numeric(drop(sv_gam_t))) {
+                score.eta.t <- der
+            }
+            else {
+                score.eta.t <- 2 * der - diag(diag(der))
+            }
+            
+           # for (k in 1:nteams) {
+           #     score.G <- bdiag(score.G, score.eta.t)
+           # }
+        
+      score.G<-ltriangle(score.eta.t)   
+                                             
+
+     if(home.field){      
+    -c(score.r,  score.G)
+    }else{
+     -c( score.G)
+    }
+}
+
 #-------------------------------
 #Data format
 
@@ -178,10 +242,28 @@ R_Y <- as.vector(R_mat$home_win)
 R_Z <- Matrix(R_RE_mat)
 t_R_Z <- t(R_Z)
 cross_R_Z <- crossprod(R_Z)
+
+if(any(R_Y==2)){
+tie.indx<-which(R_Y==2)
+X.tie<-R_X[tie.indx,,drop=FALSE]
+Z.tie<-R_Z[tie.indx,,drop=FALSE]
+R_Y<-R_Y[-tie.indx]
+R_X<-R_X[-tie.indx,,drop=FALSE]
+R_Z<-R_Z[-tie.indx,,drop=FALSE]
+for(i in 1:length(tie.indx)){
+R_Y<-c(R_Y,1,0)
+R_X<-rBind(R_X,X.tie[i,,drop=FALSE],X.tie[i,,drop=FALSE])
+R_Z<-rBind(R_Z,Z.tie[i,,drop=FALSE],Z.tie[i,,drop=FALSE])
+}
+t_R_Z <- t(R_Z)
+cross_R_Z <- crossprod(R_Z)
+Nr <- length(R_Y)
+}
+
 #initialize parameters
 eta.hat <- trc.y1 <- numeric(n_eta)
 var.eta.hat <- Matrix(0, n_eta, n_eta)
-G <- Diagonal(n_eta)
+G <- Diagonal(n_eta)   
 cons.logLik <- 0.5 * n_eta * log(2 * pi)
 #Partition desigin matrices by year and
 #calculate initial parameter values
@@ -386,8 +468,30 @@ rbetasn <- update.rbetas.first.order(eta = eta, rbetas = rbetas, R_X=R_X, R_Y=R_
     colnames(eblup) <- c("eblup", "std. error")
     rownames(eblup) <- teams
 }  #end EM
+
+
+if(home.field){
+thetas <- c( rbetas[1], G[1,1])
+names(thetas)<-c("Binary mean","G[1,1]")
+}else{
+thetas <- c(G[1,1])
+names(thetas)<-c("G[1,1]")
+}
+
+Hessian<-NULL
+if(control$Hessian){
+cat("\nCalculating Hessian with a central difference approximation...\n")
+flush.console()
+Hessian <- symmpart(jacobian(Score, thetas))
+rownames(Hessian)<-colnames(Hessian)<-names(thetas)
+#std_errors <- c(sqrt(diag(solve(Hessian))))
+if(class(try(chol(Hessian),silent=TRUE))=="try-error") cat("\nWarning: Hessian not positive-definite\n")
+}
+
+
+
             G.res<-as.matrix(G[1,1])
 colnames(G.res)<-c("Rating")
 G.res.cor<-cov2cor(G.res)
- res<-list(n.ratings.offense=NULL,n.ratings.defense=NULL,p.ratings.offense=NULL,p.ratings.defense=NULL,b.ratings=eblup[,1],n.mean=NULL,p.mean=NULL,b.mean=rbetas[1],G=G.res,G.cor=G.res.cor,R=NULL, R.cor=NULL,home.field=home.field)
+ res<-list(n.ratings.mov=NULL,n.ratings.offense=NULL,n.ratings.defense=NULL,p.ratings.offense=NULL,p.ratings.defense=NULL,b.ratings=eblup[,1],n.mean=NULL,p.mean=NULL,b.mean=rbetas[1],G=G.res,G.cor=G.res.cor,R=NULL, R.cor=NULL,home.field=home.field,Hessian=Hessian,parameters=thetas)
 }
