@@ -3,14 +3,14 @@ function(Z_mat=Z_mat, first.order=first.order, control=control,game.effect=game.
 teams <- sort(unique(c(Z_mat$home,Z_mat$away)))
 nteams <- length(teams)
 
-#first.order=TRUE produces first-order Laplace approximation,
+#first.order=TRUE produces first-order Laplace approximation,                                 
 #first.order=FALSE produces fully exponential approximation
 if(home.field&!control$OT.flag){
 j_fixed_effects <- formula(~Location+0)
 }else if(home.field&control$OT.flag){
-j_fixed_effects <- formula(~Location+as.factor(OT)+0)   
+j_fixed_effects <- formula(~Location+(OT)+0)   
 }else if(!home.field&control$OT.flag){
-j_fixed_effects <- formula(~as.factor(OT)+0) 
+j_fixed_effects <- formula(~(OT)+0) 
 }else{
 j_fixed_effects <- formula(~1)
 }
@@ -121,7 +121,7 @@ update.eta <- function(eta, J_X, J_Y, J_Z, jbetas, G, nyear, n_eta, cons.logLik)
     }
     rm(product, gr)
     chol.H <- chol(H)
-    log.p.eta <- -(length(eta)/2) * log(2 * pi) - sum(log(diag(G.chol))) - 0.5 * crossprod(eta, G.inv) %*% eta
+    log.p.eta <- -(length(eta)/2) * log(2 * pi) - sum(log(diag(G.chol))) - 0.5 * crossprod(eta, as(G.inv,"generalMatrix")) %*% eta
     log.p.j <- sum(jder0(J_X %*% jbetas + J_Z %*% eta,J_Y))
     res <- var.eta
     attr(res, "likelihood") <- as.vector(cons.logLik + log.p.eta  + log.p.j - 0.5 * (2 * sum(log(diag(chol.H)))))
@@ -240,7 +240,8 @@ cons.logLik <- 0.5 * n_eta * log(2 * pi)
 L1.conv <- FALSE
 L2.conv <- FALSE
 L1.conv.it <- 0
-jbetas<-log(as.vector(solve(crossprod(J_X))%*%t(J_X)%*%J_Y))
+suppressWarnings(jbetas<-log(as.vector(solve(crossprod(J_X))%*%t(J_X)%*%J_Y)))
+jbetas[is.na(jbetas)]<-.001
 #jbetas <- update.jbetas(eta = numeric(n_eta), var.eta = Diagonal(n_eta), jbetas = rep(23,n_jbeta), J_X=J_X, J_Y=J_Y, J_Z=J_Z, n_eta=n_eta)
 
 #these next few lines are used to populate
@@ -465,7 +466,124 @@ G.res<-as.matrix(G[1:2,1:2])
 colnames(G.res)<-c("Offense","Defense")
 G.res.cor<-cov2cor(G.res)
 
+
+Score <- function(thetas) {
+n_ybeta<-length(jbetas)
+Ny<-length(J_Y)
+    ybetas <- thetas[1:n_ybeta]
+    G <- thetas[(n_ybeta+1):length(thetas)]
+    G<-kronecker(Diagonal(length(teams)),ltriangle(G))
+   new.eta <- update.eta(eta = eta.hat, J_X = J_X, J_Y = J_Y, J_Z = J_Z,jbetas = ybetas, G = G, n_eta = n_eta, cons.logLik = cons.logLik)
+   
+    eta <- attr(new.eta, "eta")
+    eta.hat<-eta
+    var.eta <- var.eta.hat <- new.eta
+    eta.hat <- as.vector(eta)
+    temp_mat <- var.eta.hat + tcrossprod(eta.hat, eta.hat)
+   # temp_mat_R <- attr(new.eta, "h.inv") + tcrossprod(eta.hat,
+    #        eta.hat)
+    rm(new.eta)
+      score.y <- Sc.jbetas.f.first.order(eta, ybetas, J_X, J_Y, J_Z)
+    
+         gam_t_sc <- list()
+        index1 <- 0
+        score.G <- Matrix(0, 0, 0)
+         gam_t_sc <- matrix(0, 2,2)
+         index2 <- c(1)
+         for (k in 1:nteams) {
+                gam_t_sc <- gam_t_sc + temp_mat[(index2):(index2 + 
+                  1), (index2):(index2 + 1)]
+                index2 <- index2 + 2
+            }
+            gam_t <- G[1:2, 1:2]
+            sv_gam_t <- chol2inv(chol(gam_t))
+        der <- -0.5 * (nteams * sv_gam_t - sv_gam_t %*% 
+                gam_t_sc %*% sv_gam_t)
+            if (is.numeric(drop(sv_gam_t))) {
+                score.eta.t <- der
+            }
+            else {
+                score.eta.t <- 2 * der - diag(diag(der))
+            }
+            
+           # for (k in 1:nteams) {
+           #     score.G <- bdiag(score.G, score.eta.t)
+           # }
+        
+      score.G<-ltriangle(score.eta.t)    
+   
+          
+    -c(score.y,  score.G)
+}
+
+
+
+Score.ge <- function(thetas) {
+n_ybeta<-length(jbetas)
+Ny<-length(J_Y)
+    ybetas <- thetas[1:n_ybeta]
+    G <- thetas[(n_ybeta+1):length(thetas)]
+    G<-bdiag(kronecker(Diagonal(length(teams)),bdiag(ltriangle(G[1:3]))),Diagonal(Nj)*G[4])
+   new.eta <- update.eta(eta = eta.hat, J_X = J_X, J_Y = J_Y, J_Z = J_Z,jbetas = ybetas, G = G, n_eta = n_eta, cons.logLik = cons.logLik)
+   
+    eta <- attr(new.eta, "eta")
+    eta.hat<-eta
+    var.eta <- var.eta.hat <- new.eta
+    eta.hat <- as.vector(eta)
+    temp_mat <- var.eta.hat + tcrossprod(eta.hat, eta.hat)
+   # temp_mat_R <- attr(new.eta, "h.inv") + tcrossprod(eta.hat,
+    #        eta.hat)
+    rm(new.eta)
+      score.y <- Sc.jbetas.f.first.order(eta, ybetas, J_X, J_Y, J_Z)
+    
+         gam_t_sc <- list()
+        index1 <- 0
+        score.G <- Matrix(0, 0, 0)
+         gam_t_sc <- matrix(0, 2,2)
+         index2 <- c(1)
+         for (k in 1:nteams) {
+                gam_t_sc <- gam_t_sc + temp_mat[(index2):(index2 + 
+                  1), (index2):(index2 + 1)]
+                index2 <- index2 + 2
+            }
+         #gam_t_sc<-bdiag(gam_t_sc,))   
+            gam_t <- G[1:2, 1:2]
+            sv_gam_t <- chol2inv(chol(gam_t))
+        der <- -0.5 * (nteams * sv_gam_t - sv_gam_t %*% 
+                gam_t_sc %*% sv_gam_t)
+            if (is.numeric(drop(sv_gam_t))) {
+                score.eta.t <- der
+            }
+            else {
+                score.eta.t <- 2 * der - diag(diag(der))
+            }
+          der.g <- as.numeric(-0.5 * (Nj * solve(G[n_eta,n_eta]) - solve(G[n_eta,n_eta]) * 
+                sum(diag(temp_mat)[(2*nteams+1):n_eta] * solve(G[n_eta,n_eta]))))
   
-   res<-list(n.ratings.offense=NULL,n.ratings.defense=NULL,p.ratings.offense=eblup[seq(1,2*nteams,by=2),1],p.ratings.defense=eblup[seq(2,2*nteams,by=2),1],
-   b.ratings=NULL,n.mean=NULL,p.mean=jbetas,b.mean=NULL,G=G.res,G.cor=G.res.cor,R=NULL,R.cor=NULL,home.field=home.field)
+        score.G<-c(ltriangle(score.eta.t),der.g)   
+   
+          
+    -c(score.y,  score.G)
+}
+Hessian<-NULL
+thetas <- c(jbetas, reduce.G(G))
+if(control$Hessian){
+cat("\nCalculating Hessian with a central difference approximation...\n")
+flush.console()
+
+names(thetas)<-c(colnames(J_X),"G[1,1]","G[2,1]","G[2,2]")
+if(game.effect) names(thetas)<-c(colnames(J_X),"G[1,1]","G[2,1]","G[2,2]","G[3,3]")
+if(game.effect){
+Hessian <- symmpart(jacobian(Score.ge, thetas))
+}else{
+Hessian <- symmpart(jacobian(Score, thetas))
+}
+rownames(Hessian)<-colnames(Hessian)<-names(thetas)
+#std_errors <- c(sqrt(diag(solve(Hessian))))
+if(class(try(chol(Hessian),silent=TRUE))=="try-error") cat("\nWarning: Hessian not positive-definite\n")
+}
+
+  
+   res<-list(n.ratings.mov=NULL,n.ratings.offense=NULL,n.ratings.defense=NULL,p.ratings.offense=eblup[seq(1,2*nteams,by=2),1],p.ratings.defense=eblup[seq(2,2*nteams,by=2),1],
+   b.ratings=NULL,n.mean=NULL,p.mean=jbetas,b.mean=NULL,G=G.res,G.cor=G.res.cor,R=NULL,R.cor=NULL,home.field=home.field,Hessian=Hessian,parameters=thetas)
 }
